package com.csc.library.factory;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Vector;

import com.csc.library.gson.ArrayDeserializer;
import com.csc.library.gson.ByteBufferDeserialize;
import com.csc.library.gson.SerializeAndDeserializeBase;
import com.csc.library.properties.HandlePackage;
import com.csc.library.properties.JsonFile;
import com.csc.library.properties.JsonFileImpl;
import com.csc.library.request.CommonModeParameter;
import com.csc.library.request.CommonParameterImpl;
import com.csc.library.request.ReqParameterImpl;
import com.csc.library.request.RequestParameter;
import com.csc.library.request.RequestPropertiesImpl;
import com.csc.library.request.RequestWebsocket;
import com.csc.library.response.ResDataImpl;
import com.csc.library.response.ResponseData;
import com.csc.library.response.ResponsePropertiesImpl;
import com.csc.library.response.ResponseWebsocket;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

public class PackageDataFactory<Session> {
	private static PackageDataFactory instance;
	private LinkedList<ArrayList> vList = new LinkedList<ArrayList>();
	private Gson gson;
	
	private PackageDataFactory(){
		GsonBuilder gb = new GsonBuilder();
		gb.registerTypeAdapter(String[].class, new ArrayDeserializer());
		gb.registerTypeAdapter(RequestWebsocket.class, new SerializeAndDeserializeBase(RequestPropertiesImpl.class));
		gb.registerTypeAdapter(RequestParameter.class, new SerializeAndDeserializeBase(ReqParameterImpl.class));
		gb.registerTypeAdapter(CommonModeParameter.class, new SerializeAndDeserializeBase(CommonParameterImpl.class));
		gb.registerTypeAdapter(ResponseWebsocket.class, new SerializeAndDeserializeBase(ResponsePropertiesImpl.class));
		gb.registerTypeAdapter(ResponseData.class, new SerializeAndDeserializeBase(ResDataImpl.class));
		
		gb.registerTypeAdapter(JsonFile.class, new SerializeAndDeserializeBase(JsonFileImpl.class));
		gb.registerTypeAdapter(ByteBuffer.class, new ByteBufferDeserialize());
		gson = gb.create();
	}
	
	public static PackageDataFactory getInstance(){
		if(instance == null){
			instance = new PackageDataFactory();
		}
		return instance;
	}

	public Gson getGson(){
		return this.gson;
	}
	
	public void handlePackageData(Session session , HandlePackage handle , String json){
		try{
			String[] list = gson.fromJson(json , String[].class);
			if(handle.isExistsRequest(session , list[0])){
				ResponseWebsocket res = gson.fromJson(list[1], ResponseWebsocket.class);
				//check complete or error  ź͡ҡ list 㹡 tag  
				handle.onResponse(session , res);
			}else{
				RequestWebsocket req = gson.fromJson(list[1], RequestWebsocket.class);
				//check host  server ǹ зͧ  server 蹷ӧҹ
				handle.onRequest(session , req);
			}
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	
	public void sendData(Session session , HandlePackage handle , RequestWebsocket req){
		if(handle.validateSendRequest(session, req)){
			handle.send(session , createJson(req.getRequestId() , req));
		}
	}
	
	public void sendData(Session session , HandlePackage handle , ResponseWebsocket res){
		if(handle.validateSendResponse(session, res)){
			handle.send(session , createJson(res.getRequestId() , res));
		}
	}
	private String createJson(String id , Object req){
		ArrayList arr = this.getList();
		arr.add(id);
		arr.add(req);
		String json = this.getGson().toJson(arr);
		this.returnList(arr);
		return json;
	}
	private synchronized ArrayList getList(){
		synchronized (vList) {
			if(vList.size() == 0){
				return new ArrayList();
			}else{
				return vList.removeFirst();
			}
		}
	}
	private synchronized void returnList(ArrayList arr){
		synchronized (vList) {
			arr.clear();
			vList.addLast(arr);
		}
	}
}
